/* ------------------------------------------------------------------ */
/*		  C h a r I n p u t L i n e . j a v a		      */
/*		  ===================================		      */
/*								      */
/*   This file contains the source of the "CharInputLine" class.      */
/* ------------------------------------------------------------------ */


import java.io.*;
import java.util.zip.*;


/**
 * This class is used to read lines from an input stream and to parse
 * the lines read. The lines read are stored into an character array.
 *
 *    @author  Hans Blom
 *
*******************/
class  CharInputLine
/******************/
{
    /*
     * Define the buffer size for reading the input data to be parsed.
     */

    private static final int  BUFFER_SIZE = 1024;

    /*
     * Define the class constances.
     */

    private char []  buffer;	    // Contains a line read.
    private byte []  read_buffer;   // A buffer for the characters read.
    private int	     nr_chars;	    // Contains the # line characters.
    private int	     nr_read;	    // Contains the # characters read.
    private int	     offset;	    // First index of line in buffer.
    private int	     bound;	    // Size of the buffer variable.
    private int	     line_bound;    // One more than last index of line.


    /**
     * Constructor.
     *
    ***************/
    CharInputLine ()
    /**************/
    {
	/*
	 * Initialise the class variables.
	 */
	 
	buffer	    = new char [2 * BUFFER_SIZE];  // Init. line buffer.
	read_buffer = new byte [BUFFER_SIZE];	   // Init. char's read.
	nr_chars    = 0;
	nr_read     = 0;
	offset	    = 0;
	bound	    = 0;
	line_bound  = 0;

    } // CharInputLine ()


    /**
     * Get the array containing an input line.
     *
     *    @return  The character array containing an input line.
     *
    *******************/
    char []  getArray ()
    /******************/
    {
	return buffer;
    }


    /**
     * Get the index of the first character from the input line.
     *
     *    @return  The first character of the input line.
     *
    ****************/
    int  getOffset ()
    /***************/
    {
	return offset;
    }


    /**
     * Get one count more than the last index in the input line read.
     *
     *    @return  One count more than the last index in the input line.
     *
    ***************/
    int  getBound ()
    /**************/
    {
	return line_bound;
    }


    /**
     * Read a line from the specified zipped input stream. Return if a
     * line could be read.
     *
     *    @param  input   The zipped input stream to read a line from.
     *
     *    @return  <CODE>true</CODE> when a line could ber read;
     *		   <CODE>false</CODE> otherwise.
     *
    *********************************************************/
    boolean  read (ZipInputStream  input)  throws  IOException
    /********************************************************/
    {
	boolean  line_read = false;    // No line read yet.
	int	 i;

	offset	   = bound;   // Start at the remainder after last line.
	line_bound = 0;

	/*
	 * Continue while an input line is not yet read. Stop when:
	 *
	 *    o An input line was found.
	 *
	 *    o EOF was detected.
	 */

	while (!line_read  &&  nr_read != -1)
	{
	    /*
	     * Parse the characters in the input line until an input
	     * line was found. If this is not the case: read new
	     * characters.
	     */

	    while (bound < nr_chars  &&  !line_read)
	    {
		/*
		 * Parse the next character in the buffer.
		 */
		 
		switch (buffer [bound++])
		{
		    case '\r' :   // Line terminated by carriage return.

			/*
			 * Do not include line terminating character and
			 * mark the line read.
			 */

			line_bound = bound - 1;
			line_read  = true;

			/*
			 * Also skip a new line following the carriage
			 * return.
			 */

			if (bound < nr_chars && buffer [bound] == '\n')
			    bound++;
			break;

		    case '\n' :    // Line terminated by new line.

			/*
			 * Do not include line terminating character and
			 * mark the line read.
			 */

			line_bound = bound - 1;
			line_read  = true;
			break;

		    default :    // Ignore other characters.

			break;

		} // switch (buffer [bound++])

	    } // Parse the next character in the buffer.

	    /*
	     * Adjust the line buffer when no input line was yet found.
	     * After this read a new buffer read to the line buffer.
	     */

	    if (!line_read)
	    {
		/*
		 * Remove the offset bu copying the characters from the
		 * offset to the start of the line buffer.
		 */
		     
		if (offset > 0)   // Offset was found: make it zero.
		{
		    /* Copy the characters, starting at the offset,
		     * until the last caracter in the buffer to the
		     * first index of the buffer.
		     */

		    i = 0;

		    while (offset + i < nr_chars)
		    {
			buffer [i] = buffer [offset + i];
			i++;
		    }

		    /*
		     * Adjust the buffer variables.
		     */

		    offset     = 0;    // Zero offset.
		    bound      = i;    // Bound by last index + 1.
		    nr_chars   = i;    // Bound equal to # char's.
		    line_bound = 0;    // Char's > offset unparsed.
		}

		/*
		 * Read new characters into the input buffer, at least
		 * when there are still characters.
		 */

		if ( (nr_read = input.read (read_buffer)) == -1 )
		{
		    /*
		     * No characters found: stop at the last character
		     * read, when such a character was found.
		     */

		    line_bound = nr_chars;
		}
		else    // There are characters read.
		{
		    /*
		     * The new characters read do not fit into the
		     * buffer: enlarge it with the # characters into the
		     * read buffer.
		     */

		    if (nr_chars + nr_read > buffer.length)
		    {
			char []  new_buffer;
			    
			/*
			 * Allocate the enlarged buffer.
			 */
			     
			new_buffer = new char [buffer.length +
					       BUFFER_SIZE];

			/*
			 * Copy the existing characters into the new
			 * buffer.
			 */
			     
			for (i = 0;  i < nr_chars;  i++)
			    new_buffer [i] = buffer [i];

			/*
			 * Make the new buffer the current buffer.
			 */

			buffer = new_buffer;
		    }

		    /*
		     * Add the characters read to the line buffer.
		     */

		    for (i = 0;  i < nr_read;  i++)
			buffer [nr_chars + i] = (char) read_buffer [i];

		    /*
		     * Enlarge the # characters in the buffer.
		     */

		    nr_chars += nr_read;
		    
		} // There are characters read.

	    } // Parse and read input characters.

	} // Continue while an input line was not yet read.

	/*
	 * Return if an input line was found. This is the case when:
	 *
	 *    o There are characters after the offset.
	 *
	 *    o There are characters read. This implies that an empty
	 *	line was found. The characters read must be parsed in
	 *	the next call.
	 */

	return (line_bound > offset  ||  nr_read != -1);

    } // read ()


    /**
     * Convert the current line in the input buffer into a string and
     * return it.
     *
     *    @return  The current line in the input buffer as a string.
     *
    *************************/
    public String  toString ()
    /************************/
    {
	return String.copyValueOf (buffer, offset, line_bound - offset);
    }


    /**
     * Return the current line in the input buffer as a string, starting
     * at the specified index.
     *
     *    @param  begin_index   The first index of the substring to
     *				return.
     *
     *    @return  The current line as a substring, starting at the
     *		   specified index.
     *
    *************************************/
    String  toSubstring (int  begin_index)
    /************************************/
    {
	return String.copyValueOf (buffer,
				   begin_index,
				   line_bound - begin_index);
    }


    /**
     * Check if the input line starts with the specified character array
     * and return the result.
     *
     *    @param  prefix   The character array to check if the input
     *			   line starts with it.
     *
     *    @return  The result of the check: <CODE>true</CODE> when the
     *		   input line starts with the specified prefix;
     *		   <CODE>false</CODE> otherwise.
     *
    ************************************/
    boolean  startsWith (char []  prefix)
    /***********************************/
    {
	int  prefix_length = prefix.length;  // Prefix len. in help var.

	/*
	 * Compare the length of the prefix with the length of the line
	 * in the line buffer.
	 */
	 
	if (prefix_length > line_bound - offset)
	{
	    /*
	     * The prefix is larger than the input line, so the input
	     * line cannot start with it.
	     */

	    return false;
	}
	else    // Prefix smaller than the line in the input buffer.
	{
	    /*
	     * Compare the characters in the prefix with the line in the
	     * input buffer. Stop when both characters are not equal: no
	     * match.
	     */

	    boolean  equal = true;
	    int      i     = 0;

	    while (i < prefix_length  &&  equal)
		if (buffer [offset + i] == prefix [i])
		    i++;
		else
		    equal = false;    // No match anymore.

	    return equal;    // Return result of comparison.
	}

    } // startsWith ()


    /**
     * Return the first index of the input line where the specified
     * characters match with the input line. If no match can be found
     * <CODE>-1</CODE> is returned.
     *
     *    @param  chars   The character array to find the first match
     *			  of.
     *
     *    @return  The index of the first matching character in the
     *		   input array, or <CODE>-1</CODE> when no match could
     *		   be found.
     *
    ****************************/
    int  indexOf (char []  chars)
    /***************************/
    {
	/*
	 * Set some help variables.
	 */
	 
	int	 chars_length = chars.length;
	int	 bound	      = line_bound - chars_length;
	int	 i	      = offset;
	boolean  equal	      = false;

	/*
	 * Walk through the input buffer until a match was found.
	 */

	while (i < line_bound  &&  !equal)
	{
	    equal = true;

	    int  j = 0;

	    /*
	     * Compare the specified characters and the input line until
	     * the end of the character array or when no match was found
	     * for the starting index "i". The comparison is stopped at
	     * the # specified characters before the end of the input
	     * line.
	     */

	    while (j < chars_length  &&  equal)
	    	if (buffer [i + j] == chars [j])
		    j++;
		else
		    equal = false;    // No match.

	    if (!equal)    // No match: go to next index in input line.
		i++;
	}

	/*
	 * Return the index for which the first match was found or "-1".
	 */

	if (equal)
	    return i;
	else
	    return -1;

    } // indexOf ()

} // class CharInputLine
