/* ------------------------------------------------------------------ */
/*		   D a t a P l o t F r a m e . j a v a		      */
/*		   ===================================		      */
/*								      */
/*   This file contains the source of the "DataPlotFrame" class. It   */
/*   is used to define the plot frame when the plot window is shown.  */
/*   The managing of the plot frame is handled by an instance of the  */
/*   "DataPlotWindow" class.					      */
/* ------------------------------------------------------------------ */


import java.awt.*;


/**
 * This class is used to define the plot frame when the plot window is
 * shown. For each new plot window a new instance of this class will be
 * used to be sure that the information is the used browser is
 * completely refreshed. All instances of this class will be managed by
 * an instance of the <CODE>DataPlotWindow</CODE> class which is
 * <EM>not</EM> renewed when a new plot window will be shown. The
 * <CODE>DataPlotWindow</CODE> class instance will also handle the
 * required events and other actions by calls to the appropriate,
 * equally named, functions.
 *
 *    @see  DataPlotWindow
 *
 *    @author  Hans Blom
 *
***********************************/
class  DataPlotFrame  extends  Frame
/**********************************/
{
    /*
     * Define the class variables.
     */

    private DataPlotWindow  plot_window;   // The manager of this class.


    /**
     * Constructor.
     *
     *    @param  plot_window   The plot window manager of this class to
     *				which all events and actions will handed
     *				over.
     *
    ******************************************/
    DataPlotFrame (DataPlotWindow  plot_window)
    /*****************************************/
    {
	super ();   // Call the super method for default initialisation.

	this.plot_window = plot_window;    // Store the class manager.
    }


    /**
     * Validate the plot window frame. After the corresponding method in
     * the super class has been called the validation is handled by the
     * plot window manager of this class.
     *
     *    @see  DataPlotWindow#validate
     *
    ***********************/
    public void  validate ()
    /**********************/
    {
	super	   .validate ();    // Validation in super class.
	plot_window.validate ();    // Validation handled by manager.
    }


    /**
     * Handle an event in the plot window frame. The corresponding plot
     * window manager function is called to handle the event. When it
     * could handle the event, <CODE>true</CODE> is returned. Otherwise
     * the corresponding super method is called to handle the event and
     * its result is returned.
     *
     *    @param  event   The event to handle.
     *
     *    @return   A boolean which reflects if the event could be
     *		    handled.
     *
     *	  @see  DataPlotWindow#handleEvent
     *
    *****************************************/
    public boolean  handleEvent (Event  event)
    /****************************************/
    {
	if ( plot_window.handleEvent (event) )    // Handled by manager.
	{
	    return true;    // Event could be handled.
	}
	else    // Let the super method try to handle the event.
	{
	    return super.handleEvent (event);    // Return the result.
	}
    }


    /**
     * Handle an action event in the plot window frame. The
     * corresponding plot window manager function is called to handle
     * the action event. When it could handle the action event,
     * <CODE>true</CODE> is returned. Otherwise the corresponding super
     * method is called to handle the action event and its result is
     * returned.
     *
     *    @param  event    The action event to handle.
     *
     *    @param  object   The subject of the action event.
     *
     *    @return   A boolean which reflects if the action event could
     *		    be handled.
     *
     *	  @see DataPlotWindow#action
     *
    **************************************/
    public boolean  action (Event   event,
			    Object  object)
    /*************************************/
    {
	if ( plot_window.action (event, object) ) // Handled by manager.
	{
	    return true;    // Action event could be handled.
	}
	else    // Let the super method try to handle the action event.
	{
	    return super.action (event, object);   // Return the result.
	}
    }

} // class DataPlotFrame
