/* ------------------------------------------------------------------ */
/*		 H o s t T i t l e T a b l e . j a v a		      */
/*		 =====================================		      */
/*								      */
/*   This file contains the source of the "HostTitleTable" class.     */
/* ------------------------------------------------------------------ */


import java.util.*;


/**
 * This class is used to parse host tiles and to store them in a 
 * <CODE>Hashtable</CODE>.
 *
 *    @author  Hans Blom
 *
********************/
class  HostTitleTable
/*******************/
{
    /*
     * Define the positions of host and host titles in an input line.
     */

    private static final int  NR_INPUT_FIELDS = 2;  // # Fields in line.
    private static final int  TITLE_IND	      = 0;  // Host title index.
    private static final int  HOST_IND	      = 1;  // Host index.

    /*
     * Define the character which represents blanks in the host title.
     */

    private static final char  TO_BLANK_CHAR = '~';

    /*
     * Define the class instances.
     */

    private Hashtable  host_titles;    // Hashtable with host titles.
    private String []  input_words;    // Storage for split input lines.


    /**
     * Constructor.
     *
    ****************/
    HostTitleTable ()
    /***************/
    {
	/*
	 * Initialise the instance variables.
	 */

	host_titles = new Hashtable ();
	input_words = new String [NR_INPUT_FIELDS];
    }


    /**
     * Remove all elements from the host titles hashtable.
     *
    *************/
    void  clear ()
    /************/
    {
	host_titles.clear ();
    }


    /**
     * Parse an input line. Split the words in the input line at spaces.
     * Replace the blank representation characters by blanks and store
     * the host title into the <CODE>Hashtable</CODE> instance, using
     * the host as a key. The host is returned.
     *
     *    @param  input_line   The input line to parse.
     *
     *    @exception  Exception  Thrown after a parse error in the
     *				 input line.
     *
     *    @return  The host.
     *
    ***************************************************************/
    String  parseLine (CharInputLine  input_line)  throws  Exception
    /**************************************************************/
    {
	/*
	 * Split the input line into spaces.
	 */

	StringTools.split (input_words,
			   StringTools.SPACE_SEPARS,
			   input_line.getArray  (),
			   input_line.getOffset (),
			   input_line.getBound  ());

	/*
	 * Convert the blank representation characters in the input line
	 * into blanks and store the host title in the hashtable. The
	 * host is the key.
	 */

	host_titles.put (input_words [HOST_IND],
			 convertToBlanks (input_words [TITLE_IND]));

	/*
	 * Return the host.
	 */

	return input_words [HOST_IND];

    } // parseLine ()


    /**
     * Convert in the specified title the characters, representing
     * blanks, into blanks. The converted string is converted as a new
     * string.
     *
     *    @param  title   The title to convert.
     *
     *    @return   A new string containing the converted title.
     *
    **********************************************/
    private String  convertToBlanks (String  title)
    /*********************************************/
    {
	/*
	 * Set the title in an new allocated character array to easy
	 * conversion.
	 */

	char []  title_ar     = title.toCharArray ();
	int	 title_length = title_ar.length;

	/*
	 * Convert the blank representation characters in the title
	 * array into blanks.
	 */
	 
	for (int i = 0;  i < title_length;  i++)
	    if (title_ar [i] == TO_BLANK_CHAR)
		title_ar [i] = ' ';

	/*
	 * Allocate a new string, containing the converted character
	 * array, and return it.
	 */

	return new String (title_ar);

    } // converToBlanks ()


    /**
     * Get the host title from the title <CODE>Hashtable</CODE>,
     * belonging to the specified host key.
     *
     *    @param  host   The host key from which the title should be
     *			 obtained.
     *
     *    @return   The host title.
     *
    *************************/
    String  get (String  host)
    /************************/
    {
	return (String) host_titles.get (host);
    }

} // class HostTitleTable
