/* ------------------------------------------------------------------ */
/*		L e g e n d S c r o l l b a r . j a v a		      */
/*		=======================================		      */
/*								      */
/*   This file contains the source of the "LegendScrollbar" class.    */
/* ------------------------------------------------------------------ */


import java.awt.*;


/**
 * This class defines the legend scrollbar at the left of the legend
 * canvas. It extends the <CODE>Scrollbar</CODE> by:
 *
 * <OL>
 *
 * <LI>Testing if the visible part is yes/no part of the maximum
 *     scrollbar value.</LI>
 *
 * <LI>Store current parameters such that they can be used to correct
 *     a resizing of the scrollbar.</LI>
 *
 *    @author  Hans Blom
 *
*****************************************/
class  LegendScrollbar  extends  Scrollbar
/****************************************/
{
    /*
     * Define the constants used to test if the if the visible part is
     * part of the maximum scrollbar value.
     */

    private static final int  TEST_VISIBLE =  500;    // Visible length.
    private static final int  TEST_MINIMUM =    0;    // Minimum value.
    private static final int  TEST_MAXIMUM = 1000;    // Maximum value.

    /*
     * Define the class instances.
     */

    private int	     store_value;	       // Value at param. store.
    private int	     store_size;	       // Size at param. store.
    private boolean  need_substract_visible;   // Maximum - visible.


    /**
     * Constructor.
     *
    *****************/
    LegendScrollbar ()
    /****************/
    {
	super ();    // Call the constructor of the super class.

	setNeedSubstractVisible ();   // Test need to substract visible.
	initParameters		();   // Initialise storage parameters.
    }


    /**
     * Initialise the storage parameters. This is done when the
     * bar is created and it should be done when the scrollbar is
     * reseted.
     *
    **********************/
    void  initParameters ()
    /*********************/
    {
	store_value = 0;
	store_size  = 0;
    }


    /**
     * Store the scrollbar parameters. This method should be called
     * after a change in the scrollbar values, for instance after a
     * scroll event.
     *
    ***********************/
    void  storeParameters ()
    /**********************/
    {
	store_value = getValue ();

	if (need_substract_visible)
	    store_size  = getMaximum () + getVisible ();
	else
	    store_size  = getMaximum ();
    }


    /**
     * Adjust the scrollbar values after a change in its size.
     *
     *    @param  canvas_size    The size of the canvas to which the
     *				 scrollbar is connected.
     *
     *    @param  maximum_size   The (real) maximum size of the
     *				 scrollbar without correction for the
     *				 visible size.
     *
    *************************************/
    void  adjustValues (int  canvas_size,
			int  maximum_size)
    /************************************/
    {
	/*
	 * Store the line increment to reset it after sdjustment of the
	 * scrollbar values. Otherwise it may be lost.
	 */

	int  line_increment = getLineIncrement ();
	int  maximum;

	/*
	 * Adjust the maximum for the visible part, when needed.
	 */

	if (need_substract_visible)
	    maximum = maximum_size - canvas_size;
	else
	    maximum = maximum_size;

	/*
	 * Scale the stored scrollbar value according to the resizing of
	 * the scrollbar.
	 */

	int  value;

	if (store_size == 0)
	    value = 0;
	else
	    value = (store_value * maximum_size) / store_size;

	/*
	 * Correct truncating errors after the scaling of the scrollbar.
	 * Note that the value part may never exceed maximum_size minus
	 * canvas_size, because this is the maximum value for the
	 * Y scroll coordinate.
	 */

	if (value > maximum_size - canvas_size)
	    value = maximum_size - canvas_size;

	/*
	 * Set the adjusted values and restore the line increment.
	 */

	setValues	 (value, canvas_size, 0, maximum);
	setLineIncrement (line_increment);

    } // adjustValues ()


    /**
     * Test if the visible part of the scrollbar should be substracted
     * from the scrollbar maximum.
     *
    ***************************************/
    private void  setNeedSubstractVisible ()
    /**************************************/
    {
	/*
	 * Store the current values of the scrollbar.
	 */

	int  value   = getValue   ();
	int  visible = getVisible ();
	int  minimum = getMinimum ();
	int  maximum = getMaximum ();

	/*
	 * Set the values of the scrollbar. The scrollbar value is set
	 * to the maximum.
	 */

	setValues (TEST_MAXIMUM, TEST_VISIBLE,
		   TEST_MINIMUM, TEST_MAXIMUM);

	/*
	 * The visible part must be substracted from the maximum when
	 * the sum of maximum value and visible part is NOT larger than
	 * the scrollbar maximum.
	 */

	if (getValue () + getVisible () <= TEST_MAXIMUM)
	    need_substract_visible = false;
	else
	    need_substract_visible = true;

	/*
	 * Restore the old scrollbar values.
	 */

	setValues (value, visible, minimum, maximum);

    } // setNeedSubstractVisible ()

} // class LegendScrollbar
