/* ------------------------------------------------------------------ */
/*		  S k i p C o n n T a b l e . j a v a		      */
/*		  ===================================		      */
/*								      */
/*   This file contains the source of the "SkipConnTable" class.      */
/* ------------------------------------------------------------------ */


import java.util.*;


/**
 * This class is used to parse skipped connections and to store them in
 * a <CODE>Hashtable</CODE>. An instance of the <CODE>GenHostKey</CODE>
 * class is used to generate keys for the connection hashtable. The
 * key generater instance should be specified at the constructor.
 *
 *    @see  SkipConnTable#SkipConnTable
 *    @see  GenHostKey
 *
*******************/
class  SkipConnTable
/******************/
{
    /*
     * Define the class instances.
     */

    private Hashtable   skip_conns;	 // Hashtable with skipped conn.
    private GenHostKey	gen_host_key;    // To get host keys.
    private String []	conn_hosts;	 // Storage host connection.


    /**
     * Constructor. Specify the class instance of the host key
     * generator.
     *
     *    @param  gen_host_key   The specified host key generator.
     *
    ***************************************/
    SkipConnTable (GenHostKey  gen_host_key)
    /**************************************/
    {
	/*
	 * Set the specified host key generator.
	 */
	 
	this.gen_host_key = gen_host_key;

	/*
	 * Create the hashtable for the skipped connections.
	 */

	skip_conns = new Hashtable ();

	/*
	 * Allocate the (help) storage for the source and destination
	 * hosts defining a connection.
	 */

	conn_hosts = new String [2];
    }


    /**
     * Parse an input line. Split the words in the input line at spaces.
     * The both words of the input line are defining the source and
     * destination host of a connection to skip. Both hosts and the
     * specified net type are used to generate a host key. Into the
     * corresponding hashtable a value of <CODE>Boolean.TRUE</CODE> is
     * set to denote that the corresponding connection is indeed
     * skipped.
     *
     *    @param  input_line   The input line to parse.
     *
     *    @param  net_type     The net type for which the connection
     *			       must be skipped.
     *
     *    @exception  Exception   Thrown after a parse error in the
     *				  input line.
     *
    *******************************************/
    void  parseLine ( CharInputLine  input_line,
		      String	     net_type
		    )
		    throws Exception
    /******************************************/
    {
	/*
	 * Split the input line into spaces to set the source and
	 * destination host.
	 */

	StringTools.split (conn_hosts,
			   StringTools.SPACE_SEPARS,
			   input_line.getArray	(),
			   input_line.getOffset (),
			   input_line.getBound	());

	/*
	 * Use the source and destination hosts and the specified net
	 * type to set the corresponding connection as skipped.
	 */

	skip_conns.put (gen_host_key.get (conn_hosts, net_type),
			Boolean.TRUE);

    } // parseLine ()


    /**
     * Remove all elements from the skipped connections hashtable.
     *
    *************/
    void  clear ()
    /************/
    {
	skip_conns.clear ();
    }


    /**
     * This function returns if there are skipped connections.
     *
     *    @return   True when there are skipped connections; false
     *		    otherwise.
     *
    *********************/
    boolean  hasSkipped ()
    /********************/
    {
	return (skip_conns.size () > 0);
    }


    /**
     * Checks if the connection for the specified source and destination
     * hosts and the specified net type is skipped and returns the
     * result.
     *
     *    @param  conn_hosts   The array with the source and destination
     *			       hosts specifying the connection.
     *
     *    @param  net_type     The net type for which there should be
     *			       checked that the connection is skipped.
     *
     *    @return  True when the connection is skipped; false otherwise.
     *
    *****************************************/
    boolean  isSkipped (String []  conn_hosts,
			String	   net_type)
    /****************************************/
    {
	return skip_conns.containsKey (gen_host_key.get (conn_hosts,
							 net_type));
    }


    /**
     * Checks if the connection for the specified source and destination
     * hosts and the specified net type is skipped and returns the
     * result.
     *
     *    @param  hosts_key   The hosts key constrcuted from the hosts
     *			      forming the connection and from the net
     *			      type.
     *
     *    @return  True when the connection is skipped; false otherwise.
     *
    *************************************/
    boolean  isSkipped (String  hosts_key)
    /************************************/
    {
	return skip_conns.containsKey (hosts_key);
    }

} // class SkipConnTable
