/* ------------------------------------------------------------------ */
/*		    S t r i n g T o o l s . j a v a		      */
/*		    ===============================		      */
/*								      */
/*   This file contains the source of the "StringTools" class.	      */
/* ------------------------------------------------------------------ */


/**
 * This class contains some (static) methods concerned with strings.
 *
 *    @author  Hans Blom
 *
*****************/
class  StringTools
/****************/
{
    /**
     * Define the array with all space separators.
     */
    static final char []  SPACE_SEPARS = { ' ', '\t' };

    /*
     * Allocate the (static) array which is used to store one separator.
     */

    private static final char []  SEPAR_CHAR_AR = new char [1];


    /**
     * Split an input character array into a number of words, starting
     * at the specified begin index and stopping at the specified
     * boundary. A split is performed when one or more separation
     * characters are found. The separation characters are not included
     * in the split words.
     *
     *    @param  words		   The array to store the split words.
     *				   It is required that the #&nbsp;words
     *				   to split is exactly the length of
     *				   this array.
     *
     *    @param  separ_chars      The array with charcaters which are
     *				   used as separators.
     *
     *    @param  input_chars	   The input character array to split.
     *
     *    @param  input_start_ind  The index to start the split.
     *
     *    @param  input_bound	   The split is stopped at the index
     *				   <CODE>input_bound-1</CODE>.
     *
     *    @exception  Exception    An exception is thrown when the
     *				   #&nbsp;splitted words does not fit
     *				   the split words array length.
     *
    *************************************************************/
    static void  split (String []  words,
			char   []  separ_chars,
			char   []  input_chars,
			int	   input_start_ind,
			int	   input_bound)  throws  Exception
    /************************************************************/
    {
	/*
	 * Create a character array from the input string to split and
	 * initialise some help variables.
	 */

	int	 words_length = words.length;
	int	 separ_length = separ_chars.length;
	int	 input_ind    = input_start_ind;
	int	 word_ind     = 0;
	int	 word_count   = 0;
	int	 offset_ind   = 0;
	char	 input_char;
	boolean  separ_found;
	int	 separ_ind;

	/*
	 * Walk through the input character array until the end is
	 * reached.
	 */

	while (input_ind < input_bound)
	{
	    /*
	     * Seek the next separator character.
	     */

	    input_char  = input_chars [input_ind];
	    separ_found = false;
	    separ_ind	= 0;

	    while (!separ_found  &&  separ_ind < separ_length)
		if (input_char == separ_chars [separ_ind++])
		    separ_found = true;

	    if (separ_found)    // Separator between words found.
	    {
		/*
		 * if "word_count" is larger than zero a new word is
		 * found. If not we have successive separator charaters.
		 */

		if (word_count > 0)
		{
		    if (word_ind >= words_length)    // Too many words.
		    {
			String  input_str =
			    String.copyValueOf (input_chars,
						input_start_ind,
						input_bound -
						input_start_ind);

			throw new NetDataException (
				      "Too many words (" + word_ind +
				      ") in string to split: '" +
				      input_str + "'");
		    }

		    /*
		     * Add the found word to the split word array.
		     */

		    words [word_ind++] = String.copyValueOf (
					     input_chars,
					     offset_ind,
					     word_count);

		    word_count = 0;    // Reset the word.
		}
	    }
	    else    // Add a non-separator char to the next word.
	    {
		/*
		 * Set the offset index to a new word index when this
		 * new word is started.
		 */

		if (word_count == 0)
		    offset_ind = input_ind;

		word_count++;   // Add the non-separ. char. to the word.
	    }

	    input_ind++;    // Next character in the input array.

	} // Walk through the input array.

	/*
	 * Add the last word to the split word array when it exists.
	 */

	if (word_count > 0)
	{
	    if (word_ind >= words_length)    // Too many words.
	    {
		String  input_str =
		    String.copyValueOf (input_chars,
					input_start_ind,
					input_bound - input_start_ind);

		throw new NetDataException (
			      "Too many words (" + word_ind +
			      ") in string to split: '" +
			      input_str + "'");
	    }

	    /*
	     * Add the found word to the split word array.
	     */

	    words [word_ind++] = String.copyValueOf (input_chars,
						     offset_ind,
						     word_count);
	}

	if (word_ind < words_length)    // Too few words.
	{
	    String  input_str =
		String.copyValueOf (input_chars,
				    input_start_ind,
				    input_bound - input_start_ind);

	    throw new NetDataException (
			  "Too few words (" + word_ind + ") in " +
			  "string to split: '" + input_str + "'");
	}

    } // split ()


    /**
     * Split an input string into a number of words. A split is
     * performed when one or more separation characters are found. The
     * separation characters are not included in the split words.
     *
     *    @param  words		The array to store the split words. It
     *				is required that the #&nbsp;words to
     *				split is exactly the length of this
     *				array.
     *
     *    @param  separ_chars   The array with charcaters which are used
     *				as separators.
     *
     *    @param  input_str	The input string to split.
     *
     *    @exception  Exception  An exception is thrown when the
     *				 #&nbsp;splitted words does not fit the
     *				 split wors array length.
     *
    ***********************************************************/
    static void  split (String []  words,
			char   []  separ_chars,
			String	   input_str)  throws  Exception
    /**********************************************************/
    {
	/*
	 * Convert the input string to a character array and call the
	 * corresponding "split()" function with zero offet and the
	 * string length as its boundary.
	 */

	char []  input_chars = input_str.toCharArray ();

	split (words, separ_chars, input_chars, 0, input_chars.length);

    } // split ()


    /**
     * Split an input string into a number of words. A split is
     * performed when a separation character is found. The separation
     * character is not included in the split words.
     *
     *    @param  words		The array to store the split words. It
     *				is required that the #&nbsp;words to
     *				split is exactly the length of this
     *				array.
     *
     *    @param  separ_char   The separation character.
     *
     *    @param  input_str	The input string to split.
     *
     *    @exception  Exception  An exception is thrown when the
     *				 #&nbsp;splitted words does not fit the
     *				 split wors array length.
     *
    ***********************************************************/
    static void  split (String []  words,
			char	   separ_char,
			String	   input_str)  throws  Exception
    /**********************************************************/
    {
	/*
	 * Set the separation character into the constant array for one
	 * caracter.
	 */

	SEPAR_CHAR_AR [0] = separ_char;

	/*
	 * Split the input string using the separation character.
	 */

	split (words, SEPAR_CHAR_AR, input_str);

    } // split ()


    /**
     * Returns the basename of a file.
     *
     *    @param  filename   The name of the file.
     *
     *    @return   The filename to return.
     *
    ********************************************/
    static String  getBasename (String  filename)
    /*******************************************/
    {
	int  filename_length = filename.length      ();
	int  file_index	     = filename.lastIndexOf ("/");

	while (filename_length > 1 && file_index == filename_length - 1)
	{
	    filename        = filename.substring (0, file_index);
	    filename_length = file_index--;
	}

	if (file_index < filename_length - 1)
	    file_index++;

	return filename.substring (file_index);

    } // getBasename ()

} // class StringTools
