/* ------------------------------------------------------------------ */
/*		 P l o t D a t a A p p l e t . j a v a		      */
/* 		 =====================================		      */
/*								      */
/*   This file contains the source of of the "PlotApplet" class in    */
/*   the "user.plot" package. It extends the "PlotApplet" class in    */
/*   the sense that it also supports the reading of a data file.      */
/* ------------------------------------------------------------------ */


package user.plot;    // Define the plot package.

import java.lang.*;
import java.io.*;
import java.awt.*;
import java.net.*;
import java.applet.*;
import java.util.*;


/**
 * This class contains the Applet methods connected with the
 * <CODE>PlotCanvas</CODE>. It extends the <CODE>PlotAppelet</CODE>
 * class in the sense that reading from Data is suported. The data files
 * are read in the <CODE>init()</CODE> method.
 *
 *    @see  PlotDataApplet#init
 *    @see  PlotCanvas
 *    @see  PlotAppelet
 *
 *    @author  Friedrich Schotte
 *    @author  Hans Blom
 *
************************************************/
public class  PlotDataApplet  extends  PlotApplet
/***********************************************/
{
    /*
     * Define the class instances.
     */

    private PlotCanvas  plot;	      // The plot canvas.
    private Vector      filenames;    // Vector with data files.
    private Vector      formats;      // Vector with data file formats.


    /**
     * Initialise the <CODE>PlotDataApplet</CODE>. In this method the
     * plot. Also the Applet parameters are read: they are an extension
     * of the <CODE>PlotApplet</CODE> in the sense that also data files
     * and data file formats are defined. Also the data are read from
     * the files.
     *
     *    @param  plot   The used plot canvas.
     *
     *    @see  PlotApplet#init
     *
    ***********************************/
    public void  init (PlotCanvas  plot)
    /**********************************/
    {
	/*
	 * Set the instance variables.
	 */

	this.plot      = plot;	        // The plot canvas to use.
	this.filenames = new Vector (); // Set storage for data files.
	this.formats   = new Vector (); // Set storage for data formats.

	/*
	 * Read the filename and format parameters. They are required.
	 */

	if (getParameter ("data") != null) 
	{
	    StringTokenizer  list = new StringTokenizer (
					    getParameter ("data"));  

	    while (list.hasMoreTokens ())
		filenames.addElement (list.nextToken ());
	}
	else
	    setError ("did you include a 'data' parameter?");

        if (getParameter ("format") != null) 
        {
	    StringTokenizer  list = new StringTokenizer (
					    getParameter ("format"));  

            while (list.hasMoreTokens ())
		formats.addElement (list.nextToken ());
        }
        else formats.addElement ("x,y");

	/*
	 * Call the super "init()" method for the super settings and
	 * parameters.
	 */

	super.init (plot);

	/*
	 * Read the plot data from the file(s).
	 */

	readData ();

    } // init ()

    
    /**
     * Load and scan one or more data files given as parameters.
     * This is done in sequence but should probably better done in
     * parallel threads to make better use of the network capacity.
     *
    ************************/
    private void  readData ()
    /***********************/
    {  
	plot.removeAllDataTraces ();  // clear any possibly old data

	/*
	 * Read all data files.
	 */
	
	for (int i = 0;  i < filenames.size();  i++)
	{
	    String  filename = (String) filenames.elementAt (i);
	    
	    String  format = (String) formats.elementAt (
					  i % formats.size ());
					  
	    readDataFile (filename, format);
	}
    
    } // readData ()

    
    /**
     * Load an ASCII data file from the network and read in numeric
     * data.
     *
     *    @param  filename   The specified name of the data file.
     *
     *    @param  format     The format of the data.
     *
    ********************************************/
    private void  readDataFile (String  filename,
				String  format)
    /*******************************************/
    {
	/*
	 * Show the current file name in the browser bar.
	 */

	showStatus ("loading '" + filename + "'...");
	
	/*
	 * Open the URL connected to the file name.
	 */

	URL  link;
	
	try
	{
	    link = new URL (getDocumentBase(), filename);
	} 
	catch (MalformedURLException  e)
	{
	    setError ("'" + filename + "': typing mistake?");
	    return;
	}
	
	/*
	 * Opent the input stream from the URL.
	 */

	InputStream  input;
	
	try
	{
	    input = link.openStream ();
	}
	catch (IOException  e)
	{
	    setError ("'" + filename + "': broken link. (sorry)");
	    return;
	}

	/*
	 * Scan the data using a stream tokenizer.
	 */
      
        StreamTokenizer  scan = new StreamTokenizer (input);
	
	scan.eolIsSignificant (true);
      	scan.parseNumbers     ();
        scan.commentChar      ('#');
      
	Vector           columns = new Vector          ();
	StringTokenizer  list    = new StringTokenizer (format, ",");
	
	while (list.hasMoreTokens ())
	    columns.addElement (list.nextToken ());
      
	/*
	 * Read the data lines, using the data format in the tokenizer.
	 */

	try 
	{
	    /* Skip over already read-in traces. */

	    int  starting_trace = plot.getDataSize ();
	    int  token		= 0;
	    
	    while (token != StreamTokenizer.TT_EOF)
	    {
		int  trace_no = starting_trace;
		
		Double  x   = null;
		Double  dxh = null;
		Double  dxl = null;
		Double  y   = null;
		Double  dyh = null;
		Double  dyl = null;

		Enumeration  key_list = columns.elements ();

		while (key_list.hasMoreElements ())
		{
		    String  key = (String) key_list.nextElement ();
              
		    /*
		     * If key repeats store current data point and start
		     * a new one.
		     */
		     
		    if ( key.equals("x"  ) && x   != null
		         ||
		         key.equals("dx" ) && dxh != null && dxl != null
			 ||
			 key.equals("dxh") && dxh != null
			 ||
			 key.equals("dxl") && dxl != null
			 ||
			 key.equals("y"  ) && y   != null
			 ||
			 key.equals("dy" ) && dyh != null && dyl != null
			 ||
			 key.equals("dyh") && dyh != null
			 ||
			 key.equals("dyl") && dyl != null )
		    {
			/*
			 * If no x coordinates given use line number as
			 * default scaling.
			 */
			 
			if (x == null)
			    x = new Double (scan.lineno ());
               
			if (trace_no > plot.getDataSize () - 1)
			    plot.addDataTrace (new Vector ());

			Vector  trace = plot.getDataTraceAt (trace_no);

			trace.addElement (new DataPoint (x, dxh, dxl,
							 y, dyh, dyl));
			trace_no++;

			// Keep previous x values for next data point.

			y = dyh = dyl = null;
		    }
              
		    token = scan.nextToken ();

		    if (token != StreamTokenizer.TT_NUMBER)
		    {
			break; // unexpected: emergency stop
		    }
            
		    if (key.equals ("x"))
			x = new Double (scan.nval);
		    else if (key.equals ("dx"))
			dxh = dxl = new Double (scan.nval);
		    else if (key.equals ("dxh"))
			dxh = new Double (scan.nval);
		    else if (key.equals ("dxl"))
			dxl = new Double (scan.nval);
		    else if (key.equals ("y"))
			y = new Double (scan.nval);
		    else if (key.equals ("dy"))
			dyh = dyl = new Double (scan.nval);
		    else if (key.equals ("dyh"))
			dyh = new Double (scan.nval);
		    else if (key.equals ("dyl"))
			dyl = new Double (scan.nval);
		}

		/*
		 * Save last data point.
		 */

		if ( !( x == null && dxh == null && dxl == null &&
			y == null && dyh == null && dyl == null ) )
		{
		    /*
		     * If no x coordinates given use line number as
		     * default scaling.
		     */
		     
		    if (x == null)
			x = new Double (scan.lineno());

		    if (trace_no > plot.getDataSize () - 1)
			plot.addDataTrace (new Vector ());

		    Vector  trace = plot.getDataTraceAt (trace_no);
		    
		    trace.addElement (new DataPoint(x, dxh, dxl,
						    y, dyh, dyl));
		}

		while (token != StreamTokenizer.TT_EOL &&
		       token != StreamTokenizer.TT_EOF)
		{
		    token = scan.nextToken (); // go to end of line
		}

		if (scan.lineno () % 100 == 0)
		    showStatus (filename + ": " + scan.lineno() +
				" lines..");

	    } // For all data lines.
	    
	    showStatus (filename + ": " + scan.lineno () + " lines");
	}
	catch (IOException  e)
	{
	    setError ("'" + filename + "': " + e);
	    return;
	}

	if (plot.getDataSize () == 0)
	    setError ("no XY data in '" + filename + "'");

    } // readDataFile ()
    
	  
    /**
     * Return information about the Plot Applet parameters. Also the
     * parameters of the super class <CODE>PlotApplet</CODE> are
     * included in the list.
     *
     *    @return   The parameter information.
     *
     *    @see  PlotApplet#getParameterInfo
     *
    **************************************/
    public String [][]  getParameterInfo ()
    /*************************************/
    {
	String [][]  super_param_info = super.getParameterInfo ();
	int	     paramter_offset  = 2;
	String [][]  paramter_info =
	    new String [super_param_info.length + paramter_offset][];

	String []  data_param_info =
	{ "data", "URL [URL ...]",
	  "ASCII data file(s), space or tab sep. columns " +
	  "(# comment lines)" };

	String []  format_param_info =
	{ "format", "string [string ...]",
	  "data file columns key(s), e.g. x,dx[{h,l}],y,dy[{h,l}] " +
	  "(def. x,y)" };

	for (int i = 0;  i < paramter_info.length;  i++)
	    paramter_info [paramter_offset + i] = super_param_info [i];

	return paramter_info;

    } // getParameterInfo ()

} // class PlotDataApplet
