use strict;

my $FALSE = 0;
my $TRUE  = 1;

my $PROGRAM  = basename ($0);
my $PROG_DIR = dirname	($0);

my $HOME_DIR = "$PROG_DIR\\..";
my $RSC_DIR  = "$HOME_DIR\\Tmp";

sub main
{
    my $argv = shift;

    if ($#$argv > $[  &&  $argv->[$[] eq '-run_stderr_file')
    {
	runCommand ($argv);
	exit	   (1);
    }

    my $param = Param->new ();

    $param->parseArguments ($argv);

    checkRSCDir ();
    
    my $port_intv = PortIntv->new ();

    my $loop_netserver = LoopNetserver->new ($param, $port_intv);

    $loop_netserver->run ();
}

sub runCommand
{
    my $argv = shift;

    if ($argv->[$[] eq '-run_stderr_file')
    {
	shift (@$argv);

	my $stderr_file = shift (@$argv);

	open (STDERR, ">$stderr_file")  ||
	    die "$PROGRAM: $!: Redirect stderr to : $stderr_file\n";
    }

    chdir ($RSC_DIR)  ||  die "$PROGRAM: $!: Change directory: $RSC_DIR\n";

    system (@$argv);

    exit (0);
}

sub checkRSCDir
{
    if (! -d $RSC_DIR)
    {
	mkdir ($RSC_DIR)  ||
	    die "$PROGRAM: $!: Create RSC directory: $RSC_DIR\n";
    }
}

sub basename
{
    my $path = shift;

    $path =~ s/\\*$//;

    my @path_parts = split (/\\/, $path);

    return $path_parts [$#path_parts];
}

sub dirname
{
    my $path = shift;

    $path =~ s/\\*$//;

    if ($path =~ s/\\[^\\]*$$//)
    {
	return $path;
    }
    else
    {
	return '.';
    }
}

package Param;

use vars qw ( $DEBUG_KEY
	      $LOG_FILE_KEY );

$DEBUG_KEY    = 'debug';
$LOG_FILE_KEY = 'log_file';

sub new
{
    my $class = shift;

    my $self = {};

    $self->{$DEBUG_KEY	 } = $FALSE;
    $self->{$LOG_FILE_KEY} = '-';

    return bless ($self, $class);
}

sub parseArguments
{
    my $self = shift;
    my $argv = shift;

    while (my $arg = shift (@$argv))
    {
	if ($arg eq '-help'  ||  $arg eq '-?')
	{
	    printUsage ( );
	    exit       (1);
	}
	elsif ($arg eq '-debug')
	{
	    $self->{$DEBUG_KEY} = $TRUE;
	}
	elsif ($arg eq '-config_file')
	{
	    $self->parseConfigFile (shift (@$argv));
	}
	elsif ($arg eq '-log_file')
	{
	    $self->{$LOG_FILE_KEY} = getParseFileName (shift (@$argv));
	}
    }
}

sub parseConfigFile
{
    my $self	    = shift;
    my $config_file = shift;

    $config_file = getParseFileName ($config_file);

    local *config_read;

    open (config_read, "<$config_file")  ||
	die "$PROGRAM [$$]: $!: Open config file for reading: " .
	    "$config_file\n";

    my $config_expr = undef;
    my $line_nr     = 0;

    while (<config_read>)
    {
	$line_nr++;
	chomp;

	s/#.*$//;
	s/^\s+//;
	s/\s+$//;

	if (/=/)
	{
	    if (defined ($config_expr))
	    {
		$self->parseConfigExpr ($config_expr, $line_nr - 1);
	    }

	    $config_expr = $_;
	}
	elsif (!/^\s*$/)
	{
	    if (defined ($config_expr))
	    {
		$config_expr .= " $_";
	    }
	    else
	    {
		$config_expr = $_;
	    }
	}
    }

    if (defined ($config_expr))
    {
	$self->parseConfigExpr ($config_expr, $line_nr);
    }

    close (config_read)  ||
	die "$PROGRAM [$$]: $!: Close config file after reading: " .
	    "$config_file\n";
}

sub parseConfigExpr
{
    my $self	    = shift;
    my $config_expr = shift;
    my $line_nr     = shift;

    my ( $name, $value ) = split (/\s*=\s*/, $config_expr);

    if (!defined ($value))
    {
	die "$PROGRAM [$$]: Bad configuation expression at line " .
	    "#$line_nr: $config_expr\n";
    }

    $self->parseArguments ( [ "-$name", split (/\s+/, $value) ] );
}

sub getParseFileName
{
    my $file_name = shift;

    $file_name =~ s/\$HOME/$HOME_DIR/;
    $file_name =~ s/\$PROGRAM/$PROGRAM/;

    return $file_name;
}

sub printUsage
{
    print STDERR
	  "Usage: $PROGRAM [<option> [...]]\n",
	  "\n",
	  "Possible options:\n",
	  "\n",
	  "    {-help, -?}\n",
	  "          Print this message and exit.\n",
	  "\n",
	  "    -debug\n",
	  "          Print extra log debug information\n",
	  "\n",
	  "    -config_file {<config_file>,-}\n",
	  "          Specify a configuration file to supply the ",
	  "program\n",
	  "          parameters. They may be given in the format:\n",
	  "\n",
	  "              <name> = <value>\n",
	  "\n",
	  "          where <name> is one of the program options ",
	  "without\n",
	  "          leading '-'. The <value> fields my be spread ",
	  "across multiple\n",
	  "          lines. No line escape character is used. Lines ",
	  "with spaces\n",
	  "          and comment text after '#' are ignored. When '-' ",
	  "is\n",
	  "          specified, Standard Input will be used. The ",
	  "following ",
	  "          variables are substituted by:\n",
	  "\n",
	  "              \$HOME      The sup directory of this ",
	  "script.\n",
	  "\n",
	  "              \$PROGRAM   The base program name.\n",
	  "\n",
	  "    -log_file {<log_file>,-}\n",
	  "          Specify a log file. The same variables as in ",
	  "the\n",
	  "          \"-config_file\" option may be used.\n";
}

sub getDebug
{
    my $self = shift;

    return $self->{$DEBUG_KEY};
}

sub useLogFile
{
    my $self = shift;

    my $log_file = $self->{$LOG_FILE_KEY};

    return (defined ($log_file)  &&  $log_file ne '-');
}

sub getLogFile
{
    my $self = shift;

    return $self->{$LOG_FILE_KEY};
}

package PortIntv;

use vars qw ( $DFT_SYS_VALUE
	      $DEFAULT_VALUE
	      $MIN_VALUE
	      $RSC_FILE

	      $CURR_VALUE_KEY
	      $VALUE_INCR_KEY );

my $SYS_DFT_VALUE = 12865;
my $DEFAULT_VALUE = $SYS_DFT_VALUE + 11;
my $MIN_VALUE	  = 1024;
my $RSC_FILE	  = "$RSC_DIR\\netperf.port";

$CURR_VALUE_KEY = 'curr_value';
$VALUE_INCR_KEY = 'value_incr';

sub new
{
    my $class = shift;

    my $self = {};

    bless ($self, $class);

    $self->setFirst ();

    return $self;
}

sub setFirst
{
    my $self = shift;

    my $curr_value;

    if (-f $RSC_FILE)
    {
	$curr_value = getReadRSC ();
    }
    else
    {
	$curr_value = $DEFAULT_VALUE;
    }

    if ($curr_value < $MIN_VALUE)
    {
	$curr_value = $MIN_VALUE;
    }

    writeRSC ($curr_value);

    $self->{$CURR_VALUE_KEY} = $curr_value;

    $self->setIncr	();
}

sub setIncr
{
    my $self = shift;

    my $curr_value = $self->{$CURR_VALUE_KEY};


    $self->{$VALUE_INCR_KEY} = ( $curr_value < $DEFAULT_VALUE ? 1 : -1 );
}

sub getValue
{
    my $self = shift;

    return $self->{$CURR_VALUE_KEY};
}

sub setNextValue
{
    my $self = shift;

    my $curr_value = $self->{$CURR_VALUE_KEY} + $self->{$VALUE_INCR_KEY};

    if ($curr_value < $MIN_VALUE)
    {
	$curr_value		 = $MIN_VALUE;
	$self->{$VALUE_INCR_KEY} = 1;
    }

    writeRSC ($curr_value);

    $self->{$CURR_VALUE_KEY} = $curr_value;
}

sub getReadRSC
{
    local *rsc_read;

    open (rsc_read, "<$RSC_FILE")  ||
	die "$PROGRAM: $!: Open port RSC file for reading: $RSC_FILE\n";

    my $value = undef;

    while (<rsc_read>)
    {
	chomp ($value = $_);
    }

    close (rsc_read)  ||
	    die "$PROGRAM: $!: Close RSC port file after reading: $RSC_FILE\n";

    if ( !($value =~ /^\d+$/) )
    {
	$value = $DEFAULT_VALUE;
    }

    return $value;
}

sub writeRSC
{
    my $value = shift;

    local *rsc_write;

    open (rsc_write, ">$RSC_FILE")  ||
	die "$PROGRAM: $!: Open RSC port file for writing: $RSC_FILE\n";

    print rsc_write "$value\n";

    close (rsc_write)  ||
	die "$PROGRAM: $!: Close RSC port file after writing: $RSC_FILE\n";
}

package LoopNetserver;

use vars qw ( $MAX_NR_ERRORS
	      $MAX_NR_LOG_FILE_LINES
	      $CHECK_LOG_INTV

	      $PARAM_KEY
	      $PORT_INTV_KEY );

$MAX_NR_ERRORS	       = 200;
$MAX_NR_LOG_FILE_LINES = 5000;
$CHECK_LOG_INTV	       =   20;

$PARAM_KEY     = 'param';
$PORT_INTV_KEY = 'port_intv';

sub new
{
    my $class	  = shift;
    my $param	  = shift;
    my $port_intv = shift;

    my $self = {};

    $self->{$PARAM_KEY	  } = $param;
    $self->{$PORT_INTV_KEY} = $port_intv;

    return bless ($self, $class);
}

sub run
{
    my $self = shift;

    my $param	  = $self->{$PARAM_KEY	  };
    my $port_intv = $self->{$PORT_INTV_KEY};

    my $use_log_file = $param->useLogFile ();

    if ($use_log_file)
    {
	$self->openLog ();
    }

    my $check_log_count = $CHECK_LOG_INTV - 1;
    my $nr_errors       = 0;

    for (;;)
    {
	my $exit_code = $self->runNetserver ();

	if ($exit_code == 0)
	{
	    $nr_errors = 0;

	    $port_intv->setIncr ();
	}
	else
	{
	    if (++$nr_errors == $MAX_NR_ERRORS)
	    {
		die "$PROGRAM: Too many errors (= $MAX_NR_ERRORS) while " .
		    "starting netserver\n";
	    }

	    $port_intv->setNextValue ();
	}

	if ($use_log_file  &&  ++$check_log_count >= $CHECK_LOG_INTV)
	{
	    $self->closeLog	    ();
	    $self->checkLogFileSize ();
	    $self->openLog	    ();

	    $check_log_count = 0;
	}
    }
}

sub runNetserver
{
    my $self = shift;

    my $param	  = $self->{$PARAM_KEY	  };
    my $port_intv = $self->{$PORT_INTV_KEY};

    my $tmp_error_file = "$RSC_DIR\\$PROGRAM.err";

    my $port = $port_intv->getValue ();

    my @netserver = ( 'perl', '-w', $0,
		      '-run_stderr_file', $tmp_error_file,
		      'netserver', '-p', $port );

    print "Running: @netserver\n";

    my $sys_exit_code = system (@netserver) / 256;

    if ($sys_exit_code != 0)
    {
	die "$PROGRAM: Bon-zero exit code ($sys_exit_code) while " .
	    "starting in the system: @netserver\n";
    }

    my $exit_code;

    if (-s $tmp_error_file)
    {
	$exit_code = 1;

	printFile (*STDERR{IO}, $tmp_error_file);
	unlink	  ($tmp_error_file);
    }
    else
    {
	$exit_code = 0;
    }

    if ($exit_code == 0)
    {
	print "Netserver succeeded\n";
    }
    else
    {
	print "Netserver failed\n";
    }

    return $exit_code;
}

sub getComPath
{
    my $com_name = shift;

    if (-x $com_name)
    {
	return $com_name;
    }
    elsif (-x "$com_name.exe")
    {
	return "$com_name.exe";
    }

    foreach my $dir ( split (/;/, $ENV{'PATH'}) )
    {
	my $com_path = "$dir\\$com_name";

	if (-x $com_path)
	{
	    return $com_path;
	}
	elsif (-x "$com_path.exe")
	{
	    return "$com_path.exe";
	}
    }

    die "$PROGRAM: Can't find exec. in PATH: $com_name\n";
}

sub printFile
{
    my $write_descr = shift;
    my $file	    = shift;

    local *file_read;

    open (file_read, "<$file")  ||
	die "$PROGRAM: $!: Open file for reading: $file\n";

    while (<file_read>)
    {
	print $write_descr $_;
    }

    close (file_read)  ||
	die "$PROGRAM: $!: Close file after reading: $file\n";
}

sub errorReport
{
    print STDERR "$PROGRAM: ", Win32::FormatMessage ( Win32::GetLastError() );
}

sub checkLogFileSize
{
    my $self = shift;

    my $param = $self->{$PARAM_KEY};

    if (!$param->useLogFile ())
    {
	return;
    }

    my $log_file = $param->getLogFile ();

    local *log_read;

    open (log_read, "<$log_file")  ||
	die "$PROGRAM [$$]: $!: Open log to check file for reading: " .
	    "$log_file\n";

    my $nr_log_file_lines;

    while (<log_read>)
    {
	$nr_log_file_lines++;
    }

    close (log_read)  ||
	die "$PROGRAM [$$]: $!: Close log file to check after " .
	    "reading: $log_file\n";

    if ($nr_log_file_lines > $MAX_NR_LOG_FILE_LINES)
    {
	my $old_log_file = "$log_file.old";

	rename ($log_file, $old_log_file)  ||
	    die "$PROGRAM: $!: Rename $log_file to $old_log_file\n";
    }
}

sub openLog
{
    my $self = shift;

    my $log_file = $self->{$PARAM_KEY}->getLogFile ();

    open (STDOUT, ">>$log_file")  ||
	die "$PROGRAM: $!: Redirect append stdout to log file: " .
	    "$log_file\n";

    open (STDERR, ">&STDOUT")  ||
	die "$PROGRAM: $!: Redirect stderr to stdout\n";

    my $old_descr = select (STDOUT);  $| = 1;

    select (STDERR    );  $| = 1;
    select ($old_descr);
}

sub closeLog
{
    my $self = shift;

    close (STDOUT)  ||  die "$PROGRAM: $!: Close log stdout\n";
    close (STDERR)  ||  die "$PROGRAM: $!: Close log stderr\n";
}

package main;

main (\@ARGV);
